==> elevarm_ui-0.3.1.tar.gz: lib/src/advanced_components/otp_fields/elevarm_otp_field.dart <==
import 'dart:async';

import 'package:elevarm_ui/elevarm_ui.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

/// Elevarm UI's OTP Form component widget.
class ElevarmOtpField extends StatefulWidget {
  /// Callback when OTP's value is changed
  final Function(String) onChanged;

  /// Callback when resend button is pressed.
  final VoidCallback onResendOtpCode;

  /// Length of text fields
  final int length;

  /// Resend code timer duration in seconds
  final int duration;

  const ElevarmOtpField(
      {Key? key,
      required this.onChanged,
      required this.length,
      required this.onResendOtpCode,
      this.duration = 30})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => _PinTextFormField();
}

class _PinTextFormField extends State<ElevarmOtpField> {
  /// List of otp string
  late List<String> pins;

  /// List of text fields focus nodes
  late List<FocusNode> focusNodes;

  /// To show resend button text
  bool isResendOtpCode = false;

  /// Timer for otp resend countdown
  Timer? countdownTimer;

  /// Duration for timer
  late Duration timerDuration;

  @override
  void initState() {
    timerDuration = Duration(seconds: widget.duration);
    pins = List.generate(widget.length, (index) => '');
    focusNodes = List.generate(widget.length, (index) => FocusNode());
    super.initState();
    startTimer();
  }

  @override
  void dispose() {
    countdownTimer?.cancel();
    super.dispose();
  }

  /// Start OTP Resend countdown timer
  void startTimer() {
    countdownTimer =
        Timer.periodic(const Duration(seconds: 1), (_) => _setCountDown());
  }

  /// Stop OTP Resend countdown timer
  void stopTimer() {
    setState(() => countdownTimer!.cancel());
  }

  /// Reset OTP Resend countdown timer
  void resetTimer() {
    stopTimer();
    setState(() => timerDuration = Duration(seconds: widget.duration));
  }

  /// Set countdown timer text
  void _setCountDown() {
    const reduceSecondsBy = 1;
    setState(() {
      final seconds = timerDuration.inSeconds - reduceSecondsBy;
      if (seconds < 0) {
        resetTimer();
        isResendOtpCode = true;
      } else {
        timerDuration = Duration(seconds: seconds);
      }
    });
  }

  /// Generate Text Fields
  List<Widget> _generateTextFields() {
    return List.generate(widget.length, (index) => _buildTextField(index));
  }

  /// Convert `List<String>` of OTP to `String`
  void _onOtpValueChanged() {
    widget.onChanged(pins.join());
  }

  /// Handle focus changed on OTP TextFields
  void _onFocusChanged({
    required String value,
    required int indexOfTextField,
  }) {
    //only change focus to the next textField if the value entered has a length greater than one
    if (value.isNotEmpty) {
      // if the textField in focus is not the last textField,
      // change focus to the next textField
      if (indexOfTextField + 1 != widget.length) {
        //change focus to the next textField
        FocusScope.of(context).requestFocus(focusNodes[indexOfTextField + 1]);
      } else {
        //if the textField in focus is the last textField, unFocus after text changed
        focusNodes[indexOfTextField].unfocus();
      }
    } else {
      if (indexOfTextField - 1 != -1) {
        FocusScope.of(context).requestFocus(focusNodes[indexOfTextField - 1]);
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return RawKeyboardListener(
      focusNode: FocusNode(),
      onKey: (event) {
        try {
          // only get key pressed down event
          if (event.runtimeType.toString() != 'RawKeyDownEvent') {
            return;
          }

          // only get backspace key
          if (event.logicalKey != LogicalKeyboardKey.backspace) {
            return;
          }

          // get current focus index
          final focusIndex =
              focusNodes.indexWhere((element) => element.hasFocus);

          if (focusIndex == -1) {
            return;
          }

          // get prev focus index
          final prevFocusIndex = focusIndex - 1;

          if (prevFocusIndex == -1) {
            return;
          }

          // unfocus current focus
          focusNodes[focusIndex].unfocus();
          // request focus for prev focus
          FocusScope.of(context).requestFocus(focusNodes[prevFocusIndex]);
        } catch (err) {
          if (kDebugMode) {
            print(err);
          }
        }
      },
      child: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          SizedBox(
            height: 44,
            width: double.infinity,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: _generateTextFields(),
            ),
          ),
          const SizedBox(height: 16),
          Text(
            'Belum menerima kode?',
            style: ElevarmFontFamilies.inter(
              fontSize: ElevarmFontSizes.sm,
              color: ElevarmColors.neutral300,
            ),
          ),
          const SizedBox(height: 16),
          isResendOtpCode
              ? GestureDetector(
                  onTap: () {
                    setState(() {
                      isResendOtpCode = false;
                    });
                    startTimer();
                    widget.onResendOtpCode();
                  },
                  child: Text(
                    'Kirim ulang',
                    style: ElevarmFontFamilies.inter(
                        fontSize: ElevarmFontSizes.sm,
                        color: ElevarmColors.primary,
                        fontWeight: ElevarmFontWeights.semibold),
                  ),
                )
              : RichText(
                  text: TextSpan(
                    children: [
                      TextSpan(
                        text: 'Kirim kode dalam ',
                        style: ElevarmFontFamilies.inter(
                          fontSize: ElevarmFontSizes.sm,
                          color: ElevarmColors.neutral300,
                        ),
                      ),
                      TextSpan(
                        text: '${timerDuration.inSeconds.remainder(60)} detik',
                        style: ElevarmFontFamilies.inter(
                          fontSize: ElevarmFontSizes.sm,
                          fontWeight: ElevarmFontWeights.semibold,
                          color: ElevarmColors.neutral300,
                        ),
                      )
                    ],
                  ),
                ),
        ],
      ),
    );
  }

  Widget _buildTextField(int index) {
    return Flexible(
      child: Padding(
        padding: const EdgeInsets.symmetric(horizontal: 3),
        child: TextFormField(
          focusNode: focusNodes[index],
          maxLength: 1,
          keyboardType: TextInputType.number,
          textAlign: TextAlign.center,
          onChanged: (value) {
            setState(() {
              pins[index] = value;
            });
            _onOtpValueChanged();
            _onFocusChanged(
              value: value,
              indexOfTextField: index,
            );
          },
          style: ElevarmFontFamilies.inter(
            fontSize: ElevarmFontSizes.md,
            color: ElevarmColors.neutral500,
          ),
          decoration: const InputDecoration(
            counterText: '',
            contentPadding: EdgeInsets.all(10.0),
            border: OutlineInputBorder(),
            enabledBorder: OutlineInputBorder(
              borderSide: BorderSide(
                color: ElevarmColors.neutral100,
              ),
            ),
          ),
        ),
      ),
    );
  }
}

==> elevarm_ui-0.8.1.tar.gz: lib/src/advanced_components/otp_fields/elevarm_otp_field.dart <==
import 'dart:async';

import 'package:elevarm_ui/elevarm_ui.dart';
import 'package:elevarm_ui/src/styles/styles.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

/// Elevarm UI's OTP Form component widget.
class ElevarmOtpField extends StatefulWidget {
  /// Callback when OTP's value is changed
  final Function(String) onChanged;

  /// Callback when resend button is pressed.
  final VoidCallback onResendOtpCode;

  /// Length of text fields
  final int length;

  /// Resend code timer duration in seconds
  final int duration;

  const ElevarmOtpField({
    Key? key,
    required this.onChanged,
    required this.length,
    required this.onResendOtpCode,
    this.duration = 30,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _PinTextFormField();
}

class _PinTextFormField extends State<ElevarmOtpField> {
  /// List of otp string
  late List<String> pins;

  /// List of text fields focus nodes
  late List<FocusNode> focusNodes;

  /// To show resend button text
  bool isResendOtpCode = false;

  /// Timer for otp resend countdown
  Timer? countdownTimer;

  /// Duration for timer
  late Duration timerDuration;

  @override
  void initState() {
    timerDuration = Duration(seconds: widget.duration);
    pins = List.generate(widget.length, (index) => '');
    focusNodes = List.generate(widget.length, (index) => FocusNode());
    super.initState();
    startTimer();
  }

  @override
  void dispose() {
    countdownTimer?.cancel();
    super.dispose();
  }

  /// Start OTP Resend countdown timer
  void startTimer() {
    countdownTimer =
        Timer.periodic(const Duration(seconds: 1), (_) => _setCountDown());
  }

  /// Stop OTP Resend countdown timer
  void stopTimer() {
    setState(() => countdownTimer!.cancel());
  }

  /// Reset OTP Resend countdown timer
  void resetTimer() {
    stopTimer();
    setState(() => timerDuration = Duration(seconds: widget.duration));
  }

  /// Set countdown timer text
  void _setCountDown() {
    const reduceSecondsBy = 1;
    setState(() {
      final seconds = timerDuration.inSeconds - reduceSecondsBy;
      if (seconds < 0) {
        resetTimer();
        isResendOtpCode = true;
      } else {
        timerDuration = Duration(seconds: seconds);
      }
    });
  }

  /// Generate Text Fields
  List<Widget> _generateTextFields() {
    return List.generate(widget.length, (index) => _buildTextField(index));
  }

  /// Convert `List<String>` of OTP to `String`
  void _onOtpValueChanged() {
    widget.onChanged(pins.join());
  }

  /// Handle focus changed on OTP TextFields
  void _onFocusChanged({
    required String value,
    required int indexOfTextField,
  }) {
    //only change focus to the next textField if the value entered has
    //a length greater than one
    if (value.isNotEmpty) {
      // if the textField in focus is not the last textField,
      // change focus to the next textField
      if (indexOfTextField + 1 != widget.length) {
        //change focus to the next textField
        FocusScope.of(context).requestFocus(focusNodes[indexOfTextField + 1]);
      } else {
        //if the textField in focus is the last textField,
        //unFocus after text changed
        focusNodes[indexOfTextField].unfocus();
      }
    } else {
      if (indexOfTextField - 1 != -1) {
        FocusScope.of(context).requestFocus(focusNodes[indexOfTextField - 1]);
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return RawKeyboardListener(
      focusNode: FocusNode(),
      onKey: (event) {
        try {
          // only get key pressed down event
          if (event.runtimeType.toString() != 'RawKeyDownEvent') {
            return;
          }

          // only get backspace key
          if (event.logicalKey != LogicalKeyboardKey.backspace) {
            return;
          }

          // get current focus index
          final focusIndex =
              focusNodes.indexWhere((element) => element.hasFocus);

          if (focusIndex == -1) {
            return;
          }

          // get prev focus index
          final prevFocusIndex = focusIndex - 1;

          if (prevFocusIndex == -1) {
            return;
          }

          // unfocus current focus
          focusNodes[focusIndex].unfocus();
          // request focus for prev focus
          FocusScope.of(context).requestFocus(focusNodes[prevFocusIndex]);
        } catch (err) {
          if (kDebugMode) {
            print(err);
          }
        }
      },
      child: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          SizedBox(
            height: 44,
            width: double.infinity,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: _generateTextFields(),
            ),
          ),
          const SizedBox(height: 16),
          Text(
            'Belum menerima kode?',
            style: LibraryTextStyles.interSmMediumNeutral300,
          ),
          const SizedBox(height: 16),
          isResendOtpCode
              ? GestureDetector(
                  onTap: () {
                    setState(() {
                      isResendOtpCode = false;
                    });
                    startTimer();
                    widget.onResendOtpCode();
                  },
                  child: Text(
                    'Kirim ulang',
                    style: LibraryTextStyles.interSmSemiboldPrimary,
                  ),
                )
              : RichText(
                  text: TextSpan(
                    children: [
                      TextSpan(
                        text: 'Kirim kode dalam ',
                        style: LibraryTextStyles.interSmMediumNeutral300,
                      ),
                      TextSpan(
                        text: '${timerDuration.inSeconds} detik',
                        style: LibraryTextStyles.interSmSemiboldNeutral400,
                      )
                    ],
                  ),
                ),
        ],
      ),
    );
  }

  Widget _buildTextField(int index) {
    return Flexible(
      child: Padding(
        padding: const EdgeInsets.symmetric(horizontal: 3),
        child: TextFormField(
          focusNode: focusNodes[index],
          maxLength: 1,
          keyboardType: TextInputType.number,
          textAlign: TextAlign.center,
          onChanged: (value) {
            setState(() {
              pins[index] = value;
            });
            _onOtpValueChanged();
            _onFocusChanged(
              value: value,
              indexOfTextField: index,
            );
          },
          style: LibraryTextStyles.interMdRegularNeutral,
          decoration: const InputDecoration(
            counterText: '',
            contentPadding: EdgeInsets.all(10.0),
            border: OutlineInputBorder(),
            enabledBorder: OutlineInputBorder(
              borderSide: BorderSide(
                color: ElevarmColors.neutral100,
              ),
            ),
          ),
        ),
      ),
    );
  }
}
